package healthcheck

import (
	"context"
	"fmt"
	"net"
)

// TCPCheck Defines a generic TCP check
type TCPCheck struct {
	*module
}

func (hc *TCPCheck) check(baseCtx context.Context, targetIP string, targetPort int) (passed bool, err error) {
	var dialer net.Dialer
	ctx, cancel := context.WithTimeout(baseCtx, hc.Timeout)
	defer cancel()

	targetAddr := fmt.Sprintf("%s:%d", targetIP, targetPort)
	conn, err := dialer.DialContext(ctx, "tcp", targetAddr)
	if err != nil {
		return false, err
	}
	defer conn.Close()

	deadline, _ := ctx.Deadline()
	if err := conn.SetDeadline(deadline); err != nil {
		return false, err
	}

	if hc.Send == "" && hc.Expect == nil {
		return true, nil
	}

	if _, err = conn.Write([]byte(hc.Send)); err != nil {
		return false, err
	}
	var res = make([]byte, 1e6)

	if _, err := conn.Read(res); err != nil {
		return false, err
	}
	return hc.Expect.Match(res), nil
}
